#ifndef DOMINODEVICE_H
#define DOMINODEVICE_H

#include <QObject>
#include <QElapsedTimer>

#include "AdcStatus.h"
#include "ModularMregDevice.h"

class CalResult;
class OneWireArrayModule;

class DominoDevice : public ModularMregDevice
{
    Q_OBJECT
public:
    DominoDevice(const DeviceIndex &index, QObject *parent = nullptr);

public Q_SLOTS:
    void startCalibration(const ConfigKey &cal_key);

protected:
    void enforceConfigConstraints(ModularDeviceConfig *config) override;

    [[nodiscard]] bool writeConfigWithPause();

protected Q_SLOTS:
    void on_deviceConnected() override;
    void on_deviceDisconnected() override;

Q_SIGNALS:
    void adcStatusUpdated(const AdcStatus &);
    void calResultChanged(const CalResult &newCalResult) const;

private:
    bool readAdc64Status();
    std::optional<CalResult> performCalibration();

protected:
    bool readStatus() override;
    bool writeConfig() override;

    AdcStatus status;
    QElapsedTimer lastStatusTimestamp;

    QSharedPointer<OneWireArrayModule> oneWireArrayModule;
};

#endif // DOMINODEVICE_H
