#include "BmnTrcSignalRouter.h"

#include <QObject>

#include "RcCountersModel.h"
#include "TrcConfigDispatcher.h"
#include "BmnTrcCore.h"
#include "BmnTrcDevice.h"
#include "BmnTrcDeviceGlobal.h"
#include "base/DeviceId.h"
#include "configuration-manager/ConfigurationManager.h"
#include "fsm/AdcDeviceMgrFsm.h"
#include "mldiscover/DeviceDispatcher.h"
#include "lib-common/ReadoutAppConfigDb.h"
#include "pnp-server/PnpDiscovery.h"
#include "remote-control-server/RemoteControlServer.h"


#ifdef QT_WIDGETS_LIB
#include "device-discover/DiscoverDialog.h"
#include "configuration-manager/ConfigurationManager.h"
#include "m-stream-output/MStreamStatisticDialog.h"

#include "BmnTrcMainWindow.h"
#include "BmnTrcDeviceChModel.h"
#endif

static constexpr const auto device_filter_list = {
    DEVICE_ID_TRC
};

BmnTrcSignalRouter::BmnTrcSignalRouter()
{
    setDeviceIdFilter(device_filter_list);
    setConfigDispatcher(&TrcConfigDispatcher::instance());
}

void BmnTrcSignalRouter::setCore(BmnTrcCore *p)
{
    core = p;
    setReadoutAppCore(p);
}

void BmnTrcSignalRouter::connect_ConfigurationDispatcher() const
{
    ReadoutAppSignalRouter::connect_ConfigurationDispatcher();

    auto trcConfigDispatcher = &TrcConfigDispatcher::instance();

    if (deviceDispatcher)
        deviceDispatcher->subscribeDevices(trcConfigDispatcher);

    if(core) {
        trcConfigDispatcher->subscribeConfig(core);
        trcConfigDispatcher->subscribeCS(core);
    }
    if(configDb) {
        trcConfigDispatcher->subscribeConfig(configDb);
        trcConfigDispatcher->subscribeCS(configDb);
    }
    if(configurationManager)
        trcConfigDispatcher->subscribeCS(configurationManager);

#ifdef QT_WIDGETS_LIB
    for(auto m : rcCountersModels) {
        connect(trcConfigDispatcher, &TrcConfigDispatcher::configChanged,
                m, &RcCountersModel::setConfig);
    }
#endif
}

#ifdef QT_WIDGETS_LIB
void BmnTrcSignalRouter::setMainWindow(BmnTrcMainWindow *p)
{
    // set mainWindow must be before core
    mainWindow = p;
    setReadoutAppMainWindow(p);
}

void BmnTrcSignalRouter::connect_MainWindow() const
{
    ReadoutAppSignalRouter::connect_MainWindow();

    if(!core)
        return;

    if(!mainWindow)
        return;

    connect(mainWindow, &BmnTrcMainWindow::performLocalStart,
            core, &BmnTrcCore::performLocalStart);

    connect(mainWindow, &BmnTrcMainWindow::performLocalStop,
            core, &BmnTrcCore::performLocalStop);

    LldpDiscovery::subscribe(discoverDialog, device_filter_list);
    deviceDispatcher->subscribeDevices(discoverDialog);

    Q_ASSERT(deviceMgrFsm);
    connect(deviceMgrFsm, &BaseFsm::stateChanged,
            mainWindow, &BmnTrcMainWindow::onFsmStateChange);

    connect(mainWindow, &BmnTrcMainWindow::printFsmInputs,
            deviceMgrFsm, &BaseFsm::printFsmInput);
}
#endif


void BmnTrcSignalRouter::connect_RemoteControlServer() const
{
    ReadoutAppSignalRouter::connect_RemoteControlServer();

    if(!remoteControlServer)
        return;

    if (core) {
        connect(remoteControlServer, &RemoteControlServer::startRequest,
                core, &BmnTrcCore::remoteStartRequest);
        connect(remoteControlServer, &RemoteControlServer::stopRequest,
                core, &BmnTrcCore::remoteStopRequest);
        connect(remoteControlServer, &RemoteControlServer::connectionStatusUpdated,
                core, &BmnTrcCore::onRemoteStatusUpdated);

        connect(core, &BmnTrcCore::sendRunError,
                remoteControlServer,
                QOverload<const RcRunId &, QString>::of(&RemoteControlServer::onRunError));

        connect(core, &BmnTrcCore::sendRunWarning,
                remoteControlServer, &RemoteControlServer::onRunWarning);
    }
}

void BmnTrcSignalRouter::connect_TrcDeviceGlobal(const TrcDeviceGlobal *devGlob)
{
    if(!devGlob)
        return;

    auto dev = devGlob->getDevice<TrcDevice>();
    ReadoutAppSignalRouter::connect_device(devGlob, dev);

    connect(dev, &TrcDevice::statusUpdated,
            core, &BmnTrcCore::deviceStatusUpdated);

    connect(dev, &TrcDevice::statusUpdated,
            devGlob, &TrcDeviceGlobal::deviceStatusUpdated);

    connect(core, &BmnTrcCore::preRunReset,
            dev, &TrcDevice::preRunReset);

    connect(core, &BmnTrcCore::mstreamResetAll,
            devGlob, &TrcDeviceGlobal::resetMStream);

#ifdef QT_WIDGETS_LIB
    if(mainWindow){
        connect(core, &ReadoutAppCore::setStatus,
                mainWindow, &ReadoutAppMainWindow::setStatus);

        connect(dev, &TrcDevice::statusUpdated,
                mainWindow, &BmnTrcMainWindow::deviceStatusUpdated);

        connect(mainWindow, &BmnTrcMainWindow::softReset,
                dev, &TrcDevice::resetAll);

        connect(mainWindow, &BmnTrcMainWindow::softClear,
                dev, &TrcDevice::softClear);

        connect(mainWindow, &BmnTrcMainWindow::mstream_dump_reset_all,
                devGlob, &TrcDeviceGlobal::resetMStream);
    }
    for(auto m : rcCountersModels) {
        connect(dev, &TrcDevice::countersUpdated,
                m, &RcCountersModel::deviceStatusUpdated);
    }
    if(deviceChModel) {
        connect(dev, &TrcDevice::countersUpdated,
                deviceChModel, &BmnTrcDeviceChModel::deviceStatusUpdated);
    }
#endif

    deviceDispatcher->subscribeHostAddressUpdates(devGlob->getDeviceIndex(),
                                                  dev);

    // TrcConfigDispatcher::deviceInfoUpdated unused yet
    auto trcConfigDispatcher = &TrcConfigDispatcher::instance();
    connect(dev, &TrcDevice::deviceInfoUpdated,
            trcConfigDispatcher, &TrcConfigDispatcher::deviceInfoUpdated);
}
