#ifndef BMNTRCDEVICECHMODEL_H
#define BMNTRCDEVICECHMODEL_H

#include <QAbstractTableModel>
#include <QModelIndex>
#include <QVariant>
#include <optional>

#include "TrcCore/PhysicalTrigger/PhysicalTriggerConfig.h"
#include "device-modules/RcCounters/RcCounters.h"

#include "BmnTrcTreeColumns.h"

class QTimer;

class BmnTrcDeviceChModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    BmnTrcDeviceChModel(QObject *parent = nullptr);
    QVariant data(const QModelIndex &index, int role) const override;
    virtual bool setData(const QModelIndex &index,
                         const QVariant &value,
                         int role = Qt::EditRole) override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override
    {
        Q_UNUSED(parent)
        return static_cast<int>(TrcTreeColumns::COL_TOTAL);
    }

    void setEnabled(bool enabled);
    void setExpertsSettings(bool showExpters);

signals:
    void phyTrigCfgChanged(const PhysicalTriggerConfig &config) const;

public slots:
    void setPhyTrigCfg(const PhysicalTriggerConfig &newConfig);
    void deviceStatusUpdated(const BaseDevCountersMap &st);

private slots:
    void postCheckStateChange();

private:
    void sendDataChanged(QVector<int> roles = QVector<int>());

    QVariant displayData(const QModelIndex &index) const;
    QVariant toolTipData(const QModelIndex &index) const;
    QVariant editData(const QModelIndex &index) const;
    QVariant itemMaxValue(const QModelIndex &index) const;
    QVariant itemEditable(const QModelIndex &index) const;
    QVariant checkStateData(const QModelIndex &index) const;

    inline TrcTreeColumns getColumnByIndex(const QModelIndex &index) const;

    std::optional<PhysicalTriggerConfig> config;
    std::optional<BaseDevCountersMap> rcCounters;
    QTimer *const delayedSendConfigTimer;
    bool enable = true;
    bool showExpters = false;
};

#endif // BMNTRCDEVICECHMODEL_H
