#ifndef BMNTRCMAINWINDOW_H
#define BMNTRCMAINWINDOW_H

#include "BmnTrcStatus.h"

#include "lib-common/ReadoutAppMainWindow.h"

namespace Ui {
class BmnTrcMainWindow;
}

class ConfigKey;
class BmnTrcCountersDialog;
class BmnTrcDeviceChModel;
struct CalibrationTriggerConfig;

class BmnTrcMainWindow : public ReadoutAppMainWindow
{
    Q_OBJECT

public:
    explicit BmnTrcMainWindow(QWidget *parent = nullptr);
    ~BmnTrcMainWindow() override;
    DeviceStatusTableViewFancy* getDeviceStatusTableView() override;

signals:
    void softReset();
    void softClear();
    void prepareForStop(quint32 stopEvNum);
    void mstream_dump_reset_all(bool doStatClear);
    void performLocalStart() const;
    void performLocalStop() const;
    void printFsmInputs() const;

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig) override;
    void deviceStatusUpdated(const BmnTrcStatus &status);

protected:
    void saveAppGuiSettings(QSettings &) const override;
    void loadAppGuiSettings(const QSettings &) override;
    void updateControlState() override;

private slots:
    void on_pushButtonStart_clicked();
    void on_pushButtonStop_clicked();

    void on_actionShow_experts_settings_toggled(bool showExperts);
    void on_actionCounters_dialog_triggered();
    void on_actionReset_MStreamDump_triggered();
    void on_actionSend_soft_reset_triggered();

    void on_spinBoxMatchWin_valueChanged(int mw);
    void on_spinBoxTrigDelay_valueChanged(int delay);
    void on_spinBoxL0Extend_valueChanged(int extend);
    void on_radioButtonOscDelayed_toggled(bool checked);
    void on_groupBoxChWaveform_toggled(bool checked);
    void on_spinBoxLatency_valueChanged(int lat);
    void on_spinBoxReadPoints_valueChanged(int points);

    void on_groupBoxCalibTrigger_cfgUpdated(const CalibrationTriggerConfig &c);

private:
    void init_connections();
    void updateGuiVal();
    void controlStateEnable(bool enable);
    bool isControlBlocked() const;

private:
    Ui::BmnTrcMainWindow *ui;
    int blockControl = 0;

    BmnTrcDeviceChModel * const trcModel;
    BmnTrcCountersDialog * const countersDialog;
};

#endif // BMNTRCMAINWINDOW_H
