#include "ConsoleReader.h"

#include <QList>
#include <QSocketNotifier>
#include <iostream>
#include <QVariant>

namespace {
struct CommandInfo {
    QStringList cmd;
    QString descr;
    void (ConsoleReader::*pt2Member)(const QString &) const = nullptr;
};

const auto COMMANDS = QList<CommandInfo>{
{{"help", "h"}, "Print available commands", &ConsoleReader::help},
{{"printStat", "stat"}, "Print available stats", &ConsoleReader::printStat},
{{"toggleStatPrint"}, "Toggle periodical status printing", &ConsoleReader::toggleStatPrint},
{{"setPath"}, "Print available stats", &ConsoleReader::setFilePath},
{{"quit"}, "Quit from the application", &ConsoleReader::quit},
};
}

ConsoleReader::ConsoleReader(QObject *parent)
    : QObject(parent)
{
    m_notifier = new QSocketNotifier(fileno(stdin), QSocketNotifier::Read, this);
    connect(this, &ConsoleReader::help, this, &ConsoleReader::printHelp);
    connect(m_notifier, &QSocketNotifier::activated,
            this, &ConsoleReader::readCommand);
}

void ConsoleReader::readCommand()
{
    std::string line;
    std::getline(std::cin, line);
    if (std::cin.eof()) {
        emit quit({});
        return;
    }
    auto str = QString::fromStdString(line);
    auto strList = str.split(' ');
    for(auto i : COMMANDS) {
        if(i.cmd.contains(strList.first())) {
            QString arg;
            if(strList.size()>1)
                arg = strList[1];
            emit (*this.*(i.pt2Member))(arg);
        }
    }
}

void ConsoleReader::printHelp() const
{
    printf("======\n");
    printf("Program can handle following commands:\n");
    for(auto i : COMMANDS) {
        auto s = QString("%1 - %2").arg(i.cmd.join(' '), i.descr).toStdString();
        printf("\t%s\n", s.c_str());
    }
    printf("======\n");
}
