#ifndef HRBCORE_H
#define HRBCORE_H

#include <QObject>
#include <QSharedPointer>

#include "base/DeviceIndex.h"
#include "mldiscover/LldpDiscovery.h"
#include "base/ConfigIndexName.h"
#include "base/RcRunId.h"
#include "pnp-server/PNPMessage.h"
#include "fsm/FsmState.h"
#include "remote-control-server/RemoteControlServerStatus.h"

#include "config/HrbAppConfig.h"
#include "hardware/HrbDeviceGlobal.h"
#include "hardware/TtbDeviceGlobal.h"

class ConfigKey;
class ConfigSelector;
class HrbSignalRouter;
class RemoteControlServer;
class HrbConfigDb;
class ProgramDescription;
class ConfigIndexName;
class DeviceIndexEnabledMap;
class DeviceDispatcher;
class EvNumChecker;

class MStreamDeviceMgrFsm;
struct HrbStatus;
struct TtbStatus;

class HrbCore : public QObject
{
    Q_OBJECT
public:
    explicit HrbCore(QObject *parent = nullptr);
    virtual ~HrbCore() override;

signals:
    void configChanged(const HrbAppConfig &config) const;
    void switch_to_config(const ConfigIndexName &conf) const;
    void allowRemoteControl(bool allow) const;

    void closeAll() const;
    void prepareForStop(quint32 stopEvNum) const;
    void closeProgram(const ProgramDescription &) const;
    void updateProgramDescription(const ProgramDescription &) const;
    void updateControlState() const;
    void preparingRun(const RcRunId &runId) const;
    void remoteCompletedStart(bool success, const RcRunId &runId) const;
    void remoteCompletedStop(bool success, const RcRunId &runId) const;
    void sendRunError(const RcRunId &runId, QString runError=QString()) const;
    void sendRunWarning(const RcRunId &runId, QString warnStr) const;
    void mstream_reset_all(int seqNum) const;

public slots:
    void setConfig(const ConfigKey &config_key, const HrbAppConfig &newConfig);
    void configSelectorChanged(const ConfigIndexName &conf);
    void performLocalStart();
    void performLocalStop();
    void remoteStartRequest(const RcRunId &runId_, const QVariant &startEvNum=0);
    void remoteStopRequest(const RcRunId &runId_, const QVariant &stopEvNum_=0);
    void onRemoteStatusUpdated(const RemoteControlServerStatus &status);
    void onFsmStateChange(const FsmState &state, const FsmState &oldState);
    void deviceDisconnected(const DeviceIndex &index = DeviceIndex());
    void devConnectedStateUpdated(const DeviceIndex &index, bool connected);
    void onRunWarning(const QString &msg);

private slots:
    void sendMetrics() const;

private:
    void initProgDescr();
    void updateProgDescrOptions();
    void updateMetricNamespace(const ConfigIndexName conf = ConfigIndexName()) const;
    void addDevice(const DeviceIndex &index);

    RcRunId runId;
    MStreamDeviceMgrFsm * const fsm;
    RemoteControlServer * const remoteControlServer;
    HrbConfigDb * const configDb;
    DeviceDispatcher * const deviceDispatcher;
    EvNumChecker * const evNumChecker;

    HrbAppConfig config;
    RemoteControlServerStatus lastRcStatus;
    ProgramDescription progDescr;
    LldpInfoCache lldpCache;
    QMap<DeviceIndex, QSharedPointer<HrbDeviceGlobal> > hrbDevices;
    QMap<DeviceIndex, QSharedPointer<TtbDeviceGlobal> > ttbDevices;
    bool lookForStop = false;
    QVariant stopEvNum;
    int mstreamSeqNum = 0;
};
#endif // HRBCORE_H
