#include "EvbGenConfigDispatcher.h"

#include <optional>
#include <QDebug>

#include "base/ConfigIndexName.h"
#include "base/util.h"

#include "EvbGenConfig.h"

class EvbGenConfigDispatcherPrivate
{
public:
    std::optional<ConfigIndexName> cacheCIN;
    EvbGenConfig cfg;
};

EvbGenConfigDispatcher::EvbGenConfigDispatcher(QObject *parent)
    : AbstractDispatcher(parent),
    d_ptr(new EvbGenConfigDispatcherPrivate)
{
    qRegisterMetaType<ConfigIndexName>();
    qRegisterMetaType<EvbGenConfig>();
}

EvbGenConfigDispatcher::~EvbGenConfigDispatcher()
{
    delete d_ptr;
}

void EvbGenConfigDispatcher::sendCache()
{
    Q_D(const EvbGenConfigDispatcher);
    if (!d->cacheCIN.has_value())
        return;
    emit switch_to_config(d->cacheCIN.value());
    emit evbGenConfigChanged(d->cfg);
    emit evbGenServerConfigChanged(d->cfg.cfgBase);
}

void EvbGenConfigDispatcher::configSelectorChanged(const ConfigIndexName &cin)
{
    Q_D(EvbGenConfigDispatcher);

    if (d->cacheCIN.has_value() && d->cacheCIN.value() == cin)
        return;
    d->cacheCIN = cin;
    emit switch_to_config(cin);
}

void EvbGenConfigDispatcher::setEvbGenConfig(const EvbGenConfig &newConfig)
{
    // if (sender())
    //     qDebug() << "app config changed by" << qObjectAncestryStr(sender());
    Q_D(EvbGenConfigDispatcher);
    if (d->cfg == newConfig)
        return;

    d->cfg = newConfig;
    emit evbGenConfigChanged(d->cfg);
    emit evbGenServerConfigChanged(d->cfg.cfgBase);
}

void EvbGenConfigDispatcher::setEvbGenServerConfig(const GenServerConfig &newConfig)
{
    // if (sender())
    //     qDebug() << "app config changed by" << qObjectAncestryStr(sender());
    Q_D(EvbGenConfigDispatcher);
    if (d->cfg.cfgBase == newConfig)
        return;

    d->cfg.cfgBase = newConfig;
    emit evbGenConfigChanged(d->cfg);
    emit evbGenServerConfigChanged(d->cfg.cfgBase);
}
