#ifndef EVBGENCORE_H
#define EVBGENCORE_H

#include "EvbGenConfig.h"
#include "lib-common/BaseAppCore.h"
#include "lib-common/MpdRawTypes.h"

#include "remote-control-server/RemoteControlServer.h"
#include "GenStartupOptions.h"

#include "EvbGenFsm.h"

class PNPServer;
class GenServer;
class QTimer;
class GenServerStatus;
class EvbGenConfigDb;
class EvbGenConfigDispatcher;

class EvbGenCore : public BaseAppCore
{
    Q_OBJECT
public:
    explicit EvbGenCore(const GenStartupOptions &start_opt, QObject *parent = nullptr);
    ~EvbGenCore();

public slots:
    virtual void setEvbGenConfig(const EvbGenConfig &newConfig);
    void performLocalStart();
    void performLocalStop();

signals:
    void evbGenConfigChanged(const EvbGenConfig &config) const;
    void switch_to_config(const ConfigIndexName &conf) const;

    void status_manager_signal(QString str);
    void config_transfer_to_rc(QByteArray config);

    //---PnPServer signals---
    void changeProgramDescription(const ProgramDescription &) const;
    void closeProgram(const ProgramDescription &) const;

    //---signals to TCP server
    void deleteGen(int id) const;
    void startGen(const RcRunId &runId, const GenServerConfig &cfg) const;

    void requestWorkCompleted() const;
    void checkPendingData();

    //---signals to Remote control server---
    void completedCmd(bool success, QString cmd, int arg);
    void completedStart(bool success, QString runIndex, int runNumber);
    void completedStop(bool success, QString runIndex, int runNumber);
    void sendState(QString state, bool read4Run);
    void onRunError(QString runIndex,int runNumber);
    void onRunError(QString runIndex,int runNumber, QString runError);
    void onRunWarning(QString runIndex,int runNumber,QString warnStr);
    void onBackPreasure(QString runIndex, int runNumber, bool backPressure);
    void evNumCheckResponse(int checkIndex, const EvNumSet &evNums);
    void runProgressChanged(quint64 minEvNum, quint64 runEvNum, int p) const;

public slots:
    void onRemoteStartRequest(const RcRunId &runId_, const QVariant &startEvNum=0);
    void onRemoteStopRequest(const RcRunId &runId_, const QVariant &stopEvNum=0);
    // void remoteStatusUpdated(const RemoteControlServerStatus &status) override;
    void onFsmStateChange(const FsmState &state, const FsmState &oldState) override;

private slots:
    void genStatusUpdated(const GenServerStatus &st);
    void init();

protected:
    bool isRun() const override;
    const BaseFsm *getFsm() const override;

private:
    void initEvbGenProgDescr();

private slots:
    void printStats();

private:
    void setClientsCount(int cnt);
    void addGenServer(int id);
    void sendEvent();
    void sendRunDocument() const;
    int random_between(int low, int high);
    bool check_config(const GenServerConfig &cfg) const;
    void add_log(QString log) const;

    EvbGenConfig config;
    ProgramDescription prog_desc;
    QTimer * statusTimer;
    EvbGenConfigDb * const configDb;

    EvbGenFsm * const fsm;
    int clientsCount = 0;

    QMap<int, GenServerStatus> getStats;
};

#endif // EVBGENCORE_H
