#include "GenServerFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.evbGen")
}

GenServerFsm::GenServerFsm(int _id, QObject *parent)
    : BaseFsm{parent}
    , id(_id)
{
    setObjectName(QString("%1_%2").arg(__func__).arg(id));
    recreateInput(new GenServerFsmInput());
}

bool GenServerFsm::isTransitionAllowed(const FsmState &toState) const
{
    switch (getState()) {
    case FsmState::Ready:
        if(toState == FsmState::Run) return true;
        break;
    case FsmState::Run:
        if(toState == FsmState::Stopping) return true;
        break;
    case FsmState::Stopping:
    {
        auto i = getInput();
        if(toState == FsmState::Ready) return i->allDone;
        break;
    }
    default:;
    }
    return BaseFsm::isTransitionAllowed(toState);
}

void GenServerFsm::setConnected(bool connected)
{
    qInfo() << __PRETTY_FUNCTION__ << id << connected;
    auto i = getInput();
    if(i->connected == connected)
        return;
    i->connected = connected;
    evaluate();
}

void GenServerFsm::setAllDone(bool done)
{
    qInfo() << __PRETTY_FUNCTION__ << id << done;
    auto i = getInput();
    if(i->allDone == done)
        return;
    i->allDone = done;
    evaluate();
}

bool GenServerFsm::evaluate()
{
    if(BaseFsm::evaluate())
        return true;

    auto i = getInput();
    if(getState() == FsmState::Error) {
        if (i->connected) {
            setState(FsmState::Ready);
            return true;
        }
    } else {
        if(!i->connected) {
            qDebug(fsm) << __PRETTY_FUNCTION__ << "Go to Error state (on disconnect)";
            setState(FsmState::Error);
            return true;
        }
    }

    return false;
}

void GenServerFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

QDebug operator<<(QDebug dbg, const GenServerFsmInput &in)
{
    dbg << BaseFsmInput(in);

    dbg << "connected:" << in.connected << "; "
        << "allDone:" << in.allDone << "; ";
    return dbg;
}
