#include "EvbGenMainWindow.h"
#include "ui_EvbGenMainWindow.h"

#include "EvbGenSignalRouter.h"
#include "EvbGenConfigDispatcher.h"

#include "base/Globals.h"
#include "remote-control-server/RemoteControlServerStatus.h"

EvbGenMainWindow::EvbGenMainWindow(QWidget *parent)
    : PnpBaseMainWindow(parent)
    , ui(new Ui::EvbGenMainWindow)
{
    ui->setupUi(this);
    setupMenu(ui->menuFile, ui->menuHelp, ui->menuOptions, ui->menuTools);

    updateGuiEnableStates();

    auto &signalRouter = EvbGenSignalRouter::instance();
    signalRouter.setMainWindow(this);
    EvbGenConfigDispatcher::instance().subscribeCS(this);
}

EvbGenMainWindow::~EvbGenMainWindow()
{
    delete ui;
}

void EvbGenMainWindow::onFsmStateChange(const FsmState &state)
{
    PnpBaseMainWindow::onFsmStateChange(state);
    updateGuiEnableStates();
}

void EvbGenMainWindow::onRunProgressChange(quint64 minEvNum, quint64 runEvNum, int p)
{
    ui->progressBar->setValue(p);
    QString f("%p%");
    if(p!=100)
        f = QString("%L1/%L2 %p%").arg(minEvNum).arg(runEvNum);
    ui->progressBar->setFormat(f);
}

void EvbGenMainWindow::onRemoteStatusUpdated(const RemoteControlServerStatus &status)
{
    if(underRemoteCtrl == status.portInUse)
        return;

    underRemoteCtrl = status.portInUse;
    updateGuiEnableStates();
}

void EvbGenMainWindow::updateGuiEnableStates() const
{
    bool isEn = curFsmState == FsmState::Error || curFsmState == FsmState::Ready;
    if(!Globals::getDebugMode())
        ui->widgetConfig->setEnabled(isEn);
    ui->pushButtonStart->setEnabled(curFsmState == FsmState::Ready && !underRemoteCtrl);
    ui->pushButtonStop->setEnabled(curFsmState == FsmState::Run && !underRemoteCtrl);
}

void EvbGenMainWindow::on_pushButtonStart_clicked()
{
    emit performLocalStart();
}

void EvbGenMainWindow::on_pushButtonStop_clicked()
{
    emit performLocalStop();
}

