#ifndef COMBINERSTATS_H
#define COMBINERSTATS_H

#include <QMap>
#include <QMetaType>
#include <QSet>

#include <optional>

#include "base/ClientIndex.h"

#include "EventMetaInfo.h"
#include "EventSizePeriod.h"
#include "OutputTcpStatus.h"

struct CombinerClientStats {
    std::optional<quint64> lastEventNum; // last EvNum gained by Combiner
    quint64 flushedSize = 0; // summed size of all events gained by Combiner
    quint64 allBlocks = 0; // total block count gained by Combiner
    quint64 eventBlocks = 0; // total event count flushed by Combiner
    quint64 statBlocks = 0; // total count of stat records flushed by Combiner
    quint64 jsonBlocks = 0; // total count of JSON records flushed by Combiner
    quint64 trashedEvents = 0; //  number of trashed evnets
    std::optional<qint64> timeDistr; // diff. creation time (in Anal) from erliest ev in ms
    int freeEvPart = 0; //  % of free events
    int preparedEvPart = 0; //  % of prepared events
    int flushedEvPart = 0; //  % of flushed events
    std::optional<quint32> lastMissingEvNum;
};

struct CombinerStats {
    CombinerStats() { clear(); period.newPeriod(); }
    double getMaxOutputDataPart() const // in %
    {
        double ret = fwDataPart;
        for(auto v : tcpStatus) {
            if(v.index == 0)
                continue;
            ret = std::max(ret, v.dataPart);
        }
        return ret;
    }
    void clear() {
        eob = 0;
        jsonBlocks = 0;
        regularEvents = 0;
        fullEvents = 0;
        partedEvents = 0;
        skipped = 0;
        monitorEvCount = 0;
        missedFileSize = 0;
        lastEventClients.clear();
        for (ClientIndex key : clientsStat.keys()) {
            auto &cs = clientsStat[key];
            cs.lastEventNum.reset();
            cs.flushedSize = 0;
            cs.allBlocks = 0;
            cs.eventBlocks = 0;
            cs.statBlocks = 0;
            cs.jsonBlocks = 0;
            cs.trashedEvents = 0;
            cs.timeDistr.reset();
            cs.lastMissingEvNum.reset();
        }
        clientNumDistr.clear();
        timeDistrCnt = 0;
        fwDataPart = 0;
        fwFullSize = 0;
        hasPendingEvent = false;
//        tcpPorts.clear();
//        tcpIsFree.clear();
    }

    quint64 eob;
    quint64 jsonBlocks;
    quint64 regularEvents;
    quint64 fullEvents;
    quint64 partedEvents;
    quint64 skipped;
    quint64 expectedEvNum;
    quint64 monitorEvCount;
    quint64 missedFileSize;
    QSet<ClientIndex> lastEventClients; // set of clientIds in last flushed event
    QMap<int, quint64> clientNumDistr; //  [clientNumInEv] = count
                                       // (for 1st level EvB: clientNumInEv=devNumInEv)
    QMap<ClientIndex, CombinerClientStats> clientsStat;

    quint32 timeDistrCnt;
    QMap<int, OutputTcpStatus> tcpStatus;
    double fwDataPart; // in %
    int fwFullSize;
    int fwPrefSize;
    bool hasPendingEvent;

    EventSizePeriod period;
    QElapsedTimer timeStamp; // updated on send stat
};
Q_DECLARE_METATYPE(CombinerStats)

#endif // COMBINERSTATS_H
