#ifndef EVBCONFIGDISPATCHER_H
#define EVBCONFIGDISPATCHER_H

#include "EvBConfig.h"
#include "base/ClientIndexEnabled.h"
#include "base/DeviceIndexEnabled.h"
#include "lib-common/BaseConfigDispatcher.h"

class EvBConfigDispatcher : public BaseConfigDispatcher
{
    Q_OBJECT
public:
    explicit EvBConfigDispatcher(QObject *parent = nullptr);
    virtual void sendCache() override;

    template<class T>
    void subscribeEvbConfig(T *obj) {
            connect(this, &EvBConfigDispatcher::configChanged,
                    obj, &T::setConfig,
                    Qt::UniqueConnection);

            connect(obj, &T::configChanged,
                    this, &EvBConfigDispatcher::setConfig,
                    Qt::UniqueConnection);
            logConnection(obj, "Config");
    }

    template<class T>
    void subscribeClientList(T *obj) {
        connect(obj, &T::clientListChanged,
                this, &EvBConfigDispatcher::setClientList,
                Qt::UniqueConnection);
        connect(this, &EvBConfigDispatcher::clientListChanged,
                obj, &T::setClientList,
                Qt::UniqueConnection);
        logConnection(obj, "ClientList");
    }

    template<class T>
    void subscribeFileSplitterConfig(T *obj) {
        connect(obj, &T::fileSplitterConfigChanged,
                this, &EvBConfigDispatcher::setFileSplitterConfig,
                Qt::UniqueConnection);
        connect(this, &EvBConfigDispatcher::fileSplitterConfigChanged,
                obj, &T::setFileSplitterConfig,
                Qt::UniqueConnection);
        logConnection(obj, "EvbFileSplitterConfig");
    }

    template<class T>
    void subscribeClientConfig(T *obj) {
        connect(obj, &T::clientConfigChanged,
                this, &EvBConfigDispatcher::setClientConfig,
                Qt::UniqueConnection);
        connect(this, &EvBConfigDispatcher::clientConfigChanged,
                obj, &T::setClientConfig,
                Qt::UniqueConnection);
        logConnection(obj, "EvBClientConfig");
    }

    template<class T>
    void subscribeCombinerConfig(T *obj) {
        connect(obj, &T::combinerConfigChanged,
                this, &EvBConfigDispatcher::setCombinerConfig,
                Qt::UniqueConnection);
        connect(this, &EvBConfigDispatcher::combinerConfigChanged,
                obj, &T::setCombinerConfig,
                Qt::UniqueConnection);
        logConnection(obj, "EvbCombinerConfig");
    }

    template<class T>
    void subscribeEvNumberConfig(T *obj) {
        connect(obj, &T::evNumberConfigChanged,
                this, &EvBConfigDispatcher::setEvbEvNumberConfig,
                Qt::UniqueConnection);
        connect(this, &EvBConfigDispatcher::evNumberConfigChanged,
                obj, &T::setEvbEvNumberConfig,
                Qt::UniqueConnection);
        logConnection(obj, "EvbEvNumberConfig");
    }

    template<class T>
    void subscribeWriterConfig(T *obj) {
        connect(obj, &T::writerConfigChanged,
                this, &EvBConfigDispatcher::setWriterConfig,
                Qt::UniqueConnection);
        connect(this, &EvBConfigDispatcher::writerConfigChanged,
                obj, &T::setWriterConfig,
                Qt::UniqueConnection);
        logConnection(obj, "EvbWriterConfig");
    }

signals: void fileSplitterConfigChanged(const EvbFileSplitterConfig &) const;
public slots: void setFileSplitterConfig(const EvbFileSplitterConfig &newConfig);

signals: void configChanged(const EvBConfig &) const;
public slots: void setConfig(const EvBConfig &newConfig);

signals: void clientListChanged(const ClientIndexEnabledMap &) const;
public slots: void setClientList(const ClientIndexEnabledMap &new_clients);

signals: void clientConfigChanged(const EvBClientConfig &) const;
public slots: void setClientConfig(const EvBClientConfig &cfg);

signals: void combinerConfigChanged(const EvbCombinerConfig &) const;
public slots: void setCombinerConfig(const EvbCombinerConfig &cfg);

signals: void evNumberConfigChanged(const EvbEvNumberConfig &) const;
public slots: void setEvbEvNumberConfig(const EvbEvNumberConfig &cfg);

signals: void writerConfigChanged(const EvbWriterConfig &) const;
public slots: void setWriterConfig(const EvbWriterConfig &cfg);

protected:
    DeviceIndexEnabledMap getDeviceIndexEnabledMap() const override { return {}; }

private:

    std::unique_ptr<EvBConfig> config;
};

#endif // EVBCONFIGDISPATCHER_H
