#include "EvbProgramDescriptionHelper.h"

#include "base/DeviceId.h"
#include "pnp-server/PNPMessage.h"

namespace {
constexpr const char *EVB_DATA_TYPE_KEY = "data_type";
constexpr const char *EVB_DATA_TYPE_NON_EVENT = "non-event";
}

bool isNonEventBaseProgram(const ProgramDescription &pd)
{
    switch(DaqModule::Type(pd.getDaqModule())) {
    case DaqModule::Type::MStream:
    {
        auto idStr = pd.options.value("dev_id");
        bool ok;
        auto id = idStr.toInt(&ok, 16);
        static const QSet<int> nonEvDevId { DEVICE_ID_MSC16VE_E,
                                          DEVICE_ID_SOFTWARE, DEVICE_ID_UT24VE};
        return ok && (nonEvDevId.contains(id));
    }
    case DaqModule::Type::EventBuilder:
    {
        auto dtStr = pd.options.value(EVB_DATA_TYPE_KEY);
        return dtStr == EVB_DATA_TYPE_NON_EVENT;
    }
    default:;
    }
    return false;
}


void markProgramAsNonEventBase(ProgramDescription &pd, bool v)
{
    if(v)
        pd.options[EVB_DATA_TYPE_KEY] = EVB_DATA_TYPE_NON_EVENT;
    else
        pd.options.remove(EVB_DATA_TYPE_KEY);
}
