#ifndef TCPCLIENTSTATUS_H
#define TCPCLIENTSTATUS_H

#include "EvbSocketStatus.h"
#include <optional>

#include "base/ClientIndex.h"
#include "pnp-server/PNPMessage.h"

struct TcpStatistic : public EvbSocketStatus
{
    TcpStatistic(ClientIndex i = ClientIndex()) : index(i) {}
    friend bool operator==(const TcpStatistic &, const TcpStatistic &) = default;

    ClientIndex index;

    std::optional<double> loopSpeed() const
    {
        auto dt = loopTimer.elapsed();
        if(dt) return 1e3*loopRecivedSize/dt;
        return {};
    }
    void nextLoopIfElapsed() {
        if(loopTimer.elapsed() < 1e3)
            return;
        loopTimer.start();
        loopRecivedSize = 0;
    }

    QElapsedTimer loopTimer;
    int loopRecivedSize = 0;
};
Q_DECLARE_METATYPE(TcpStatistic)

enum class TcpClientState {
    Disabled,
    WaitSource,
    SourceBusy,
    Connected
};

enum class FreeSpaceState {
    Empty,
    AlmostEmpty,
    Medium,
    AlmostFull,
    Full
};

struct TcpClientStatus
{
    TcpClientStatus(ClientIndex index={}, int bufSize=0)
        : index(index)
        , statistic{index}
        , preferedBufSize(bufSize)
    {}
    friend bool operator==(const TcpClientStatus &, const TcpClientStatus &) = default;

    ClientIndex index;
    TcpStatistic statistic;

    TcpClientState state = TcpClientState::WaitSource;
    FreeSpaceState freeSpaceState = FreeSpaceState::Full;
    bool waitingResize = true;
    int preferedBufSize;
    QString stateStr;
    ProgramDescription lastProgDescription;
    ProgramInterface usedInterface;
};
Q_DECLARE_METATYPE(TcpClientStatus)

#endif // TCPCLIENTSTATUS_H
