#include "EvbClientStatusTableView.h"

#include <QHeaderView>

#include "mvc/ClientEnableTableModel.h"
#include "mvc/MergeColumnsTableModel.h"
#include "EvbClientStatusModel.h"

EvbClientStatusTableView::EvbClientStatusTableView(QWidget *p)
    : QTableView(p)

    , enabledModel(new ClientEnableTableModel(this))
    , statusModel(new EvbClientStatusModel(this))
    , mergeModel(new MergeColumnsTableModel(this))
{

    mergeModel->addSourceModel(enabledModel, Qt::UserRole);
    mergeModel->addSourceModel(statusModel, Qt::UserRole);

    setModel(mergeModel);
    connect(enabledModel, &ClientEnableTableModel::clientListChanged,
            this, &EvbClientStatusTableView::clientListChanged);

    connect(this, &EvbClientStatusTableView::combinerStatsUpdated,
            statusModel, &EvbClientStatusModel::setCombinerStats);
    connect(this, &EvbClientStatusTableView::clientStatusUpdated,
            statusModel, &EvbClientStatusModel::setTcpClientStatus);
    connect(statusModel, &EvbClientStatusModel::colHiddenUpdated,
            this, &EvbClientStatusTableView::setSubModelColHidden);

    for(auto c : {ClientEnableTableModel::COL_TYPE,
                   ClientEnableTableModel::COL_INDEX})
        horizontalHeader()->hideSection(c);
    setShowAdvancedCounters(false);

    // setFont({"Monospace"});
    setSelectionBehavior(QAbstractItemView::SelectRows);
    horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

void EvbClientStatusTableView::setShowAdvancedCounters(bool showColumns)
{
    for(auto c : {EvbClientStatusModel::COL_TCP_BUFF,
                   EvbClientStatusModel::COL_EV_BUFF})
        setSubModelColHidden(c, !showColumns);
}

void EvbClientStatusTableView::setShowTimeDistrCol(bool showColumns)
{
    setSubModelColHidden(EvbClientStatusModel::COL_TIME_DISTR, !showColumns);
}

ClientIndexSet EvbClientStatusTableView::getSelectedClients() const
{
    QSet<int> selectedRows;
    const QAbstractItemModel *model = nullptr;
    const auto &selection = selectedIndexes();
    for (const auto &i : selection) {
        selectedRows.insert(i.row());
        if (!model)
            model = i.model();
        else
            Q_ASSERT(model == i.model());
    }

    ClientIndexSet clSet;
    for (auto row : selectedRows) {
        if (!model) break;
        // const auto &index = model->index(row, 1);
        // Q_ASSERT(index.isValid());
        const auto &clIndex = enabledModel->clientIndexOf(row);
        clSet.insert(clIndex);
    }
    return clSet;
}

void EvbClientStatusTableView::setControlEnabled(bool en)
{
    enabledModel->setEnabled(en);
}

void EvbClientStatusTableView::setClientList(const ClientIndexEnabledMap &newList)
{
    enabledModel->setClientList(newList);
    statusModel->setClientList(newList);
}

void EvbClientStatusTableView::setSubModelColHidden(int col, bool hidden)
{
    auto c = mergeModel->getColumnFromSource(statusModel, col);
    if(c == -1)
        return;
    setColumnHidden(c, hidden);
}

void EvbClientStatusTableView::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected)
{
    QTableView::selectionChanged(selected, deselected);

    const auto &indices = selectedIndexes();
    for (auto it = indices.begin(); it != indices.end(); ++it) {
        if(!enabledModel->clientIndexOf(it->row()).isEmpty()) {
            emit clientSelectionChanged(true);
            return;
        }
    }
    // qInfo() << __PRETTY_FUNCTION__ << clSet;
    emit clientSelectionChanged(false);
    // emit setDeviceIdFilter(id_set);
}
