#ifndef TLU40LVDSMODEL_H
#define TLU40LVDSMODEL_H

#include <QAbstractItemModel>
#include <QModelIndex>
#include <QVariant>
#include <optional>

#include "base/DeviceIndex.h"
#include "lib-common/BaseDeviceAppConfig.h"

#include "Tlu40LvdsModelItem.h"
#include "Tlu40LvdsLuminosity/Tlu40LvdsLuminosityBuildParams.h"
#include "Tlu40LvdsLuminosity/Tlu40LvdsLuminosityStatus.h"
#include "Tlu40LvdsTofTrigger/Tlu40LvdsTofTriggerBuildParams.h"
#include "Tlu40LvdsTofTrigger/Tlu40LvdsTofTriggerStatus.h"
#include "TluDecDes/TluDecDesBuildParams.h"

class QTimer;
class ConfigKey;

class Tlu40LvdsModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    Tlu40LvdsModel(QObject *parent = nullptr);

    QVariant data(const QModelIndex &index, int role) const override;
    virtual bool setData(const QModelIndex &index,
                         const QVariant &value,
                         int role = Qt::EditRole) override;
    Qt::ItemFlags flags(const QModelIndex &index) const override;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role) const override;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const override;
    QModelIndex parent(const QModelIndex &index) const override;
    int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    void setEnabled(bool en);
    int getDevRow(const DeviceIndex &di) const;

signals:
    void configChanged(const BaseDeviceAppConfig &config) const;

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig);
    void setTluDesDecBp(const DeviceIndex &index, const
                             TluDecDesBuildParams &bp);
    void setTlu40LvdsLuminosityBp(const DeviceIndex &index,
                                  const Tlu40LvdsLuminosityBuildParams &bp);
    void setTlu40LvdsTofTriggerBp(const DeviceIndex &index,
                                  const Tlu40LvdsTofTriggerBuildParams &bp);
    void setTlu40LvdsLuminosityStatus(const DeviceIndex &index,
                                      const Tlu40LvdsLuminosityStatus st);
    void setTlu40LvdsTofTriggerStatus(const DeviceIndex &index,
                                      const Tlu40LvdsTofTriggerStatus &st);
    void resetCounters();
    void onDeviceConnected(const DeviceIndex &devIndex);

private slots:
    void postCheckStateChange();
    void processPendingStatus();

private:
    void enqueueConfigSend();
    void doReset();
    void sendDataChanged(QVector<int> roles = QVector<int>(), const QModelIndex parent = {});

    QVariant displayData(const QModelIndex &index) const;
    QVariant editData(const QModelIndex &index) const;
    QVariant toolTipData(const QModelIndex &index) const;
    QVariant textAlignmentData(const QModelIndex &index) const;
    QVariant checkStateData(const QModelIndex &index) const;
    QVariant limitsData(const QModelIndex &index) const;
    bool setEditRole(const QModelIndex &index, const QVariant &value);
    bool setCheckStateData(const QModelIndex &index, const QVariant &value);

    inline Tlu40LvdsModelItem *getItemByIndex(const QModelIndex &index) const;
    QVariant getRecursiveState(const QModelIndex &mIndex) const;
    QPair<std::optional<quint32>, std::optional<quint32>> getCounterPair(Tlu40LvdsModelItem *item) const;

    QTimer * const pendingTimer;
    BaseDeviceAppConfig config;
    QMap<DeviceIndex, Tlu40LvdsLuminosityStatus> devStatusLum;
    QMap<DeviceIndex, Tlu40LvdsTofTriggerStatus> devStatusTof;
    QMap<DeviceIndex, Tlu40LvdsLuminosityStatus> devStatusOffsetLum;
    QMap<DeviceIndex, Tlu40LvdsTofTriggerStatus> devStatusOffsetTof;
    QMap<DeviceIndex, Tlu40LvdsLuminosityStatus> pendingDevStatusLum;
    QMap<DeviceIndex, Tlu40LvdsTofTriggerStatus> pendingDevStatusTof;
    QMap<DeviceIndex, Tlu40LvdsLuminosityBuildParams> devBpLum;
    QMap<DeviceIndex, Tlu40LvdsTofTriggerBuildParams> devBpTof;
    QMap<DeviceIndex, TluDecDesBuildParams> devBpDecDes;
    Tlu40LvdsModelItem rootItem;
    QTimer *const delayedSendConfigTimer;
    bool enable = true;
};

#endif // TLU40LVDSMODEL_H
