#include "MscConfigDispatcher.h"

#include "base/util.h"
#include "lib-common/BaseDeviceAppConfig.h"
#include "status/MscBuildParams.h"

#include <QDebug>

MscConfigDispatcher::MscConfigDispatcher(QObject *parent)
    : ReadoutAppConfigDispatcher(parent)
{}

void MscConfigDispatcher::setConfig(const BaseDeviceAppConfig &config)
{
    if (config == *cache) {
        return;
    }
    check_config();
    ReadoutAppConfigDispatcher::setConfig(config);
}

void MscConfigDispatcher::setDeviceConfig(const DeviceIndex &index,
                                          const ModularDeviceConfig &config)
{
    if(!cache->knownConfigs.contains(index)){
        return;
    }
    if(cache->knownConfigs[index] == config){
        return;
    }

    cache->knownConfigs[index] = config;
    qDebug() << "device config changed by" << qObjectAncestryStr(sender());

    check_config();
    incrementConfigKey();
}

void MscConfigDispatcher::setBuildParameters(const DeviceIndex &index,
                                             const MscBuildParams &b_params)
{
    if(!cache->knownConfigs.contains(index)){
        return;
    }

    auto &dev_config = cache->knownConfigs[index].mscCore;
    if(b_params.mscInput.has_value()) {
        auto iBP = b_params.mscInput.value();
        const auto chNum = iBP.channels;
        const auto hHum = iBP.hists;

        for(int i : dev_config.hist.plot.keys()){
            if(i >= hHum)
                dev_config.hist.plot.remove(i);
        }

        for(int i : dev_config.input.keys()){
            if(i >= hHum)
                dev_config.input.remove(i);
        }

        for(int i : dev_config.chThreshold.keys()){
            if(i >= chNum)
                dev_config.chThreshold.remove(i);
        }

        if(dev_config.hist.plot.size() < hHum){
            auto hist_size = static_cast<quint16>(dev_config.hist.plot.size());
            for(quint16 i=hist_size; i<hHum; ++i){
                dev_config.hist.plot[i] = MscHistPlotConfig();
                dev_config.input[i] = {.histCh=i};
            }
        }
        if(dev_config.chThreshold.size() < chNum){
            int thresh_size = dev_config.chThreshold.size();
            for(int i=thresh_size; i<chNum; ++i){
                dev_config.chThreshold[i] = 0;
            }
        }

        auto &ceMonitor = dev_config.mstream.ceMonitor;
        if(ceMonitor.size()>iBP.ceInputs)
            ceMonitor.resize(iBP.ceInputs);
        while(ceMonitor.size()<iBP.ceInputs)
            ceMonitor.push_back(false);
    }
    incrementConfigKey();
}

void MscConfigDispatcher::resetConfigHistCh()
{
    for(auto devIt=cache->knownConfigs.begin(); devIt!=cache->knownConfigs.end(); ++devIt) {
        for(auto chIt=devIt->mscCore.input.begin(); chIt!=devIt->mscCore.input.end(); ++chIt) {
            chIt->histCh = chIt.key();
        }
    }

    incrementConfigKey();
}

void MscConfigDispatcher::check_config()
{
    for(const auto index : cache->knownConfigs.keys()){
        auto &hist = cache->knownConfigs[index].mscCore.hist;
        auto max = std::numeric_limits<quint32>::max()/1000;
        for(int hId : hist.plot.keys()){
            if(hist.plot[hId].plotSlice > max)
                hist.plot[hId].plotSlice = max;
        }
    }
}

void MscConfigDispatcher::sendConfigKeyCache()
{
    auto enSet = cache->getDeviceIndexEnabledMap().getEnabled();
    if (enSet.size() > 1) {
        qWarning() << "More then one device in Msc16ve config!";
        enSet.erase(enSet.begin());
        for(auto i : enSet) {
            cache->knownConfigs.remove(i);
        }
    }

    ReadoutAppConfigDispatcher::sendConfigKeyCache();
}
