#include "PNPBeacon.h"

#include <iostream>

#include <QTime>
#include <QTimer>

namespace {
const int sendPeriod = 120;
const int PACKET_SIZE_DEFAULT = 8000;
}

PNPBeacon::PNPBeacon(QStringList args)
    : QObject(NULL)
    , sendTimer(new QTimer(this))
{
    QString index;
    if(args.size() < 2 || args[1].isEmpty())
        index = QHostInfo::localHostName();
    else
        index = args[1];
    const auto &programIndex = ProgramIndex::fromString(index);
    progDescr.setClientIndex({DaqModule(DaqModule::Type::PnpBeacon), programIndex});

    int packetSize;
    bool ok=false;
    if(args.size() > 2) {
        packetSize = args[2].toInt(&ok);
    }

    if(!ok) {
        packetSize=PACKET_SIZE_DEFAULT;
    }

    if(packetSize) {
        qInfo() << "Set packet size to " << packetSize;
        const int dataSize = packetSize - 44; // data size instead of frame size;
        qInfo() << "Data size will be " << dataSize;
        qInfo() << "size0:" << progDescr.getAnnounceStr().toLatin1().size();
        progDescr.options["payload"] = "";
        int size1 = progDescr.getAnnounceStr().toLatin1().size();
        int sizeDiff = dataSize-size1;
        qInfo() << "size1:" << size1;
        for(int i=0;i<sizeDiff;++i) {
            progDescr.options["payload"].append("a");
        }
        qInfo() << "size2:" << progDescr.getAnnounceStr().toLatin1().size();
    }

    connect(sendTimer, &QTimer::timeout, this, &PNPBeacon::sendMessage);
    sendTimer->start(1000);
    qInfo() << progDescr.getAnnounceStr();
}

PNPBeacon::~PNPBeacon()
{
    emit closeProgram(progDescr);
}

void PNPBeacon::sendMessage()
{
    emit updateProgramDescription(progDescr);
    if((sendCnt++)%sendPeriod==0)
        std::cerr << '\n'
                  << QTime::currentTime().toString().toStdString();
    std::cerr << ".";
}
