/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef VMETRIGSCONTROLLERPRIVATE_H
#define VMETRIGSCONTROLLERPRIVATE_H

#include <QObject>
#include <memory>

#include "base/ConfigKey.h"
#include "base/DeviceIndexEnabled.h"
#include "base/RcRunId.h"
#include "fsm/FsmState.h"

class DeviceDispatcher;
class RcConfigDispatcher;
struct RcConfig;
struct VmeTrigConfig;
struct VmeTrigStatus;
class VmeTrigFsm;
class VmeTrigDeviceProxy;
class VmeTrigsController;
struct ModularDeviceConfig;

class VmeTrigsControllerPrivate : public QObject
{
    Q_OBJECT
public:
    VmeTrigsControllerPrivate(VmeTrigsController *p, QObject *parent = nullptr);
    ~VmeTrigsControllerPrivate();

signals:
    void printFsmInput() const;
    void deviceConfigUpdated(const ConfigKey &key, const ModularDeviceConfig &newConfig) const;

private slots:
    void trigStatusUpdated(const DeviceIndex &index, const VmeTrigStatus &status);
    void incrementSetupKey();

public:
    DeviceIndexEnabledMap myDevices() const;
    static DeviceIndexEnabledMap deviceListFromConfig(const RcConfig &newConfig);
    void addTrigDevice(const DeviceIndexEnabled &die);
    void removeTrigDevice(const DeviceIndex &index);
    void addTrigThread(const DeviceIndex &index, const VmeTrigConfig &vc);
    void removeTrigThread(const DeviceIndex &index);
    void sendVmeMap() const;

private:
    void sendConfigToDevices() const;

    Q_DECLARE_PUBLIC(VmeTrigsController)
    VmeTrigsController * const q_ptr;
    std::unique_ptr<RcConfig> config;
    VmeTrigFsm * const fsm;
    QMap<DeviceIndex, VmeTrigDeviceProxy> trigThreads;
    QMap<DeviceIndex, VmeTrigStatus> trigStatus;

    RcRunId last_got_run_id;

    ConfigKey setupKey;
};

#endif // VMETRIGSCONTROLLERPRIVATE_H
