#ifndef TQDCCORE_H
#define TQDCCORE_H

#include <QObject>

#include "TqdcDeviceGlobal.h"
#include "base/ConfigKey.h"
#include "base/DeviceIndex.h"
#include "cal/CalConfig.h"
#include "fsm/FsmState.h"
#include "lib-common/BaseDeviceAppConfig.h"
#include "lib-common/ReadoutAppCore.h"
#include "TelnetRemoteControl/TelnetBlockedEvents.h"

class AdcDeviceMgrFsm;
class CalConfigDb;
class ConfigIndexName;
class ConfigKey;
class ConfigSelector;
class DeviceDispatcher;
class ProgramDescription;
class ReadoutAppConfigDb;
class TqdcConfigDispatcher;
class TqdcTelnetRemoteControl;

struct RemoteControlServerStatus;
struct RunLogicMultiStatus;
struct TqdcStartupOptions;

class TqdcCore : public ReadoutAppCore
{
    Q_OBJECT
public:
    explicit TqdcCore(const TqdcStartupOptions &so, QObject *parent = nullptr);

signals:
    void configChanged(const BaseDeviceAppConfig &config) const;
    void calConfigChanged(const CalConfig &) const;
    void prepareForStop(quint32 stopEvNum);
    void updateControlState();
    void stat_from_receiver_updated(const ReceiverStat &stat);
    void remoteCompletedStart(bool success, const RcRunId &runId);
    void remoteCompletedStop(bool success, const RcRunId &runId);
    void startTqdcCalibration(const ConfigKey &cal_key);
    void startAdcSrdDesCalibration(const ConfigKey &cal_key, bool hardMode, const QString &resDir) const;
    void timedRunFinished(const TelnetBlockedEvents &evBloked) const;
    void preRunCheck();

public slots:
    void setConfig(const ConfigKey &configKey, const BaseDeviceAppConfig &newConfig);
    void performLocalStart();
    void performLocalStop();
    void remoteStartRequest(const RcRunId &runId_, const QVariant &startEvNum=0);
    void remoteStopRequest(const RcRunId &runId_, const QVariant &stopEvNum=0);
    void remoteStatusUpdated(const RemoteControlServerStatus &status) override;
    void onFsmStateChange(const FsmState &state, const FsmState &oldState) override;
    void setDeviceStatus(const ModularDeviceStatus &status) override;
    void startCalibration_();
    void requestAdcSerDesCalibration(bool hardMode, const QString &dir);
    void startTelnetRun();
    void setRunLogicStatus(const RunLogicMultiStatus &);

protected:
    bool isRun() const override;
    const BaseFsm *getFsm() const override;
    BaseDeviceAppConfig getBaseAppConfig() const final { return config; }

private:
    void initTqdcProgDescr();
    void updateTqdcProgDescrOptions();
    void addDevice(const DeviceIndex &index) override;
    void removeDevice(const DeviceIndex &index) override;
    void sendRunDocument() const;

    AdcDeviceMgrFsm * const fsm;
    ReadoutAppConfigDb * const tqdcConfigDb;
    CalConfigDb * const calConfigDb;
    BaseDeviceAppConfig config;
    QMap<DeviceIndex, QSharedPointer<TqdcDeviceGlobal> > devices;
    ConfigKey cal_key;
    bool adcSerDesCalHardMode = false;
    QString adcSerDesCalResDir;
    TqdcTelnetRemoteControl * const telnetRc;
    std::optional<bool> telnetRun;
    TelnetBlockedEvents telnetRunBlockedEv;
};

#endif // TQDCCORE_H
