/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef CLIENTINDEX_H
#define CLIENTINDEX_H

#include <QMetaType>
#include <QString>

#include "DaqModule.h"
#include "ProgramIndex.h"

class QDebug;

class ClientIndex
{
public:
    ClientIndex(const DaqModule &type = DaqModule(), const ProgramIndex &index = ProgramIndex()) :
        program_type(type), program_index(index)
    {}

    bool isEmpty() const { return program_type.isEmpty() && program_index.isDefault(); }
    DaqModule getProgramType() const { return program_type; }
    ProgramIndex getProgramIndex() const { return program_index; }
    QString toString() const;
    static ClientIndex fromString(const QString &r);

    bool operator==(const ClientIndex &index) const;
    bool operator!=(const ClientIndex &index) const { return !operator==(index); }
    bool operator<(const ClientIndex &r) const;

    friend uint qHash(const ClientIndex &r, uint seed = 0)
    {
        return qHash(r.program_type, seed) ^ qHash(r.program_index, seed);
    }
    friend QDebug operator<<(QDebug dbg, const ClientIndex &r);
    friend QDataStream &operator<<(QDataStream &out, const ClientIndex &value);
    friend QDataStream &operator>>(QDataStream &in, ClientIndex &value);

    DaqModule program_type;
    ProgramIndex program_index;
};

Q_DECLARE_METATYPE(ClientIndex)

typedef QSet<ClientIndex> ClientIndexSet;
Q_DECLARE_METATYPE(ClientIndexSet)

ClientIndexSet makeClientIndexSet(const QList<ClientIndex> &list);

#endif // CLIENTINDEX_H
