#ifndef CONFIGCONVERTERUTIL_H
#define CONFIGCONVERTERUTIL_H

#include <QJsonObject>
#include <QJsonArray>
#include <QMetaType>
#include <QVariantMap>


QJsonValue jsonValueFromUInt64(const quint64 &in);
quint64 jsonValueToUInt64(const QJsonValue &in);

class ConfigConverterUtil
{
public:
    template<typename K>
    static bool update_value(const QJsonObject &in, const QString &key, K *val) {
        auto it = in.find(key);
        if(it==in.end())
            return false;
        Q_ASSERT(val);
        auto var = QVariant::fromValue(it.value());
        assert(var.canConvert<K>());
        *val = var.value<K>();
        return true;
    }

    template<class K, class V>
    static QJsonValue toJsonValue(const QMap<K, V> &data)
    {
        QJsonObject qvm;
        Q_ASSERT(QVariant::fromValue(K()).template canConvert<QString>());
        for (auto it = data.cbegin(); it != data.cend(); ++it) {
            const auto &k = QVariant::fromValue(it.key()).toString();
            const auto &jval = QVariant::fromValue(it.value())
                    .template value<QJsonValue>();
            qvm.insert(k, jval);
        }
        return qvm;
    }

    template<class K, class V>
    static QJsonValue toJsonValue(const QHash<K, V> &data)
    {
        QJsonObject qvm;
        Q_ASSERT(QVariant::fromValue(K()).template canConvert<QString>());
        for (auto it = data.cbegin(); it != data.cend(); ++it) {
            const auto &k = QVariant::fromValue(it.key()).toString();
            const auto &jval = QVariant::fromValue(it.value())
                    .template value<QJsonValue>();
            qvm.insert(k, jval);
        }
        return qvm;
    }

    template<class K, class V>
    static QMap<K, V> toMap(const QJsonValue &val)
    {
        return toMap<K, V>(val.toObject());
    }

    template<class K, class V>
    static QMap<K, V> toMap(const QJsonObject &obj)
    {
        QMap<K, V> map;
        Q_ASSERT(QVariant::fromValue(QString()).template canConvert<K>());
        for (auto it = obj.begin(); it != obj.end(); ++it) {
            const K &di = QVariant::fromValue(it.key()).value<K>();
            const V &v = QVariant::fromValue(it.value()).value<V>();
            map.insert(di, v);
        }
        return map;
    }


    template<class V>
    static QJsonValue toJsonValue(const QList<V> &data)
    {
        QJsonArray array;
        Q_ASSERT(QVariant::fromValue(V()).template canConvert<QJsonValue>());
        for (auto it = data.cbegin(); it != data.cend(); ++it) {
            const auto &jval = QVariant::fromValue(*it)
                    .template value<QJsonValue>();
            array.append(jval);
        }
        return array;
    }

    template<class V>
    static QList<V> toList(const QJsonValue &val)
    {
        return toList<V>(val.toArray());
    }

    template<class V>
    static QList<V> toList(const QJsonArray &obj)
    {
        QList<V> list;
        Q_ASSERT(QVariant::fromValue(QJsonValue()).template canConvert<V>());
        for (auto it = obj.begin(); it != obj.end(); ++it) {
            const V &v = QVariant::fromValue(*it).value<V>();
            list.append(v);
        }
        return list;
    }

#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
    template<class V>
    static QJsonValue toJsonValue(const QVector<V> &data)
    {
        QJsonArray array;
        Q_ASSERT(QVariant::fromValue(V()).template canConvert<QJsonValue>());
        for (auto it = data.cbegin(); it != data.cend(); ++it) {
            const auto &jval = QVariant::fromValue(*it)
                    .template value<QJsonValue>();
            array.append(jval);
        }
        return array;
    }
#endif

    template<class V>
    static QVector<V> toVector(const QJsonValue &val)
    {
        return toVector<V>(val.toArray());
    }

    template<class V>
    static QVector<V> toVector(const QJsonArray &obj)
    {
        QVector<V> list;
        Q_ASSERT(QVariant::fromValue(QJsonValue()).template canConvert<V>());
        for (auto it = obj.begin(); it != obj.end(); ++it) {
            const V &v = QVariant::fromValue(*it).value<V>();
            list.append(v);
        }
        return list;
    }

    template<class V>
    static std::vector<V> toStdVector(const QJsonValue &val)
    {
        auto v = ConfigConverterUtil::toVector<V>(val);
        return std::vector<V>(v.begin(), v.end());
    }

    template<class V>
    static QJsonValue fromStdVector(const std::vector<V> &in){
        return ConfigConverterUtil::toJsonValue<V>(QVector<V>(in.begin(), in.end()));
    }
};

#endif // CONFIGCONVERTERUTIL_H
