/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "gui_utils.h"

#include <QComboBox>

Qt::CheckState toCheckState(const QVariant &v)
{
    if (!v.isValid() || v.isNull() || !v.canConvert<bool>()) return Qt::PartiallyChecked;
    return v.toBool() ? Qt::Checked : Qt::Unchecked;
}

QList<QWidget *> listChildWidgetsRecursive(QWidget *p)
{
    QList<QWidget *> list;
    if (p) {
        if (p->isWidgetType()) {
            list << p;
        }
        for (auto i : p->children()) {
            if (i != p && i->isWidgetType()) {
                auto w = qobject_cast<QWidget *>(i);
                if (w)
                    list << listChildWidgetsRecursive(w);
            }
        }
    }
    return list;
}

void updateComboBox(QComboBox *cb, const ComboBoxData &contents)
{
    // find items to remove, O(n*n)
    QList<int> toRemove;
    for (int i=0; i<cb->count(); i++) {
        const auto &cbText = cb->itemText(i);
        const auto &cbData = cb->itemData(i);
        bool found = false;
        for (const auto &c : contents) {
            if (c.text == cbText && c.data == cbData) {
                found = true; break;
            }
        }
        if (!found)
            toRemove.push_back(i);
    }
    // remove items in reverse order, starting at end
    std::sort(toRemove.begin(), toRemove.end(), std::greater<>()); // reverse sort
    for (auto i : toRemove)
        cb->removeItem(i);

    // add items
    for (const auto &i : contents) {
        auto index = cb->findText(i.text);
        if (index < 0 || cb->itemData(index) != i.data) {
            cb->addItem(i.text, i.data);
        }
    }
}
