/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DSPLIB_FITTERGAUSPROFILE_H
#define DSPLIB_FITTERGAUSPROFILE_H

#include <QMetaType>

namespace dsplib {

typedef enum {
    FIT_PROFILE_FAST      = 1,
    FIT_PROFILE_OPTIMAL   = 2,
    FIT_PROFILE_PRECISION = 3,
    FIT_PROFILE_CUSTOM    = 4,
    FIT_PROFILE_SUPERFAST = 5,
    FIT_PROFILE_DEFAULT   = FIT_PROFILE_OPTIMAL
} FitProfileEnum_t;

enum {FIT_ALGO_ACCURATE=0, FIT_ALGO_COARSE=1};

inline const int default_fitside = 6;
inline const int default_fitfixpar = 0;       // bit 0: amplitude, bit1: time, bit2: width. Default: no fixed par
inline const unsigned int  default_fititer = 2; // 1 is enough. 2 for safe.
inline const int default_fitalgo = FIT_ALGO_ACCURATE;

class FitterGausProfile
{
public:
    friend bool operator==(const FitterGausProfile &, const FitterGausProfile &) = default;

    bool isValid() const;
    int get_dec_fititer() const;
    int get_dec_fitalgo() const;
    int get_dec_fitfixpar() const;
    int get_fit_half_np(double pulse_width_steps) const;

    FitProfileEnum_t fprof = FIT_PROFILE_DEFAULT; //!< Fitter profile select (superset of fit- iter, side, fixpar, algo)
    int fititer = default_fititer;                //!< Fitter iterations limit
    int fitside = 0;                              //!< Fitter number of points (2*N+1 points to fit), 0 - auto
    unsigned int fitfixpar = default_fitfixpar;   //!< bit field, select which parameter to fix
    int fitalgo = default_fitalgo;                //!< Fitter algorithm select
};

} // namespace dsplib

Q_DECLARE_METATYPE(dsplib::FitProfileEnum_t)
Q_DECLARE_METATYPE(dsplib::FitterGausProfile)

#endif // DSPLIB_FITTERGAUSPROFILE_H
