/*
**    Copyright 2021 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef JSONRPC_H
#define JSONRPC_H

// connect sendMessage / messageReceived to network transport
// connect rpcReceived / rpcRespondResult to client application
// use rpcCall / rpcComplete in server application

#include <QObject>
#include <QJsonObject>

#include "JsonRpcTypes.h"

class JsonRpcPrivate;

class JsonRpc : public QObject
{
    Q_OBJECT
public:
    explicit JsonRpc(QObject *parent = nullptr);
    ~JsonRpc();
    JsonRpcRequest findRequest(const RpcRequestId &id) const;

signals:
    void rpcReceived(const RpcRequestId &id, const JsonRpcRequest &request);
    void rpcComplete(const RpcRequestId &id, const QJsonValue &result);
    void rpcError(const RpcRequestId &id, const QJsonValue &error);
    void sendMessage(const QJsonObject &json);

public slots:
    RpcRequestId rpcCall(const JsonRpcRequest &request, int timeoutMs);
    void rpcRespondResult(const RpcRequestId &id, const QJsonValue &result);
    void rpcRespondError(const RpcRequestId &id, JsonRpcError error);
    void messageReceived(const QJsonObject &j);

protected slots:
    void onRpcError(const RpcRequestId &id, const QJsonValue &error);

private slots:
    void pollQueue();

private:
    void dequeue(const QVariant &id);

    Q_DECLARE_PRIVATE(JsonRpc)
    JsonRpcPrivate * const d_ptr;
};

#endif // JSONRPC_H
