/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ABSTRACTDEVICEMODULE_H
#define ABSTRACTDEVICEMODULE_H

#include <QtCore>

#include "device-modules/MregBusInterface.h"

class DeviceIndex;
class DeviceModuleCommand;
class DeviceModuleCommandResult;
class SdbModuleIndex;

enum DeviceId : uint8_t;
enum AFI_SDB_ID : uint32_t;
struct fw_version_t;
struct sdb_t;
namespace mlink { class RegOpVector; }

class AbstractDeviceModule
{
public:
    explicit AbstractDeviceModule(const MregBusInterface &bus);
    virtual ~AbstractDeviceModule();

    virtual void init() = 0;
    virtual void deinit() = 0;
    virtual void post_init() = 0;
    virtual DeviceModuleCommandResult runCommand(const DeviceModuleCommand &command);
    bool isAvailable() const;
    quint32 getBaseAddr() const { return bus.baseAddr(); }
    AFI_SDB_ID getSdbId() const;
    SdbModuleIndex moduleIndex() const;

    QString name() const;
    DeviceIndex getDeviceIndex() const;

protected:
    DeviceId getDeviceId() const;
    void setBaseAddr(quint16 baseAddr, quint16 mask);
    quint16 baseAddr() const;
    quint16 lastAddr() const;
    uint8_t majorVersion() const;
    fw_version_t getVersion() const;

    bool regOpExec(mlink::RegOpVector &data);
    mlink::RegOpVector regOpDumpAll(QVector<quint16> *result) const;
    bool memReadBlk(int startWord, size_t wordCount, std::vector<quint32> *result);
    bool memWriteBlk(int wordNumber, const std::vector<quint32> &data);

    MregBusInterface bus;
};

#endif // ABSTRACTDEVICEMODULE_H
