#include "AdcLemoMuxModule.h"

#include "regio/regop.h"

#include "AdcLemoMuxConfig.h"
#include "sdb/AfiSdbId.h"

enum {
    REG32_ID        = 0x0,
    REG_CONTROL     = 0x2,
    REG_OUT_NUM     = 0x4,
    REG64_NAME_BASE   = 0x10,
};

QList<AFI_SDB_ID> AdcLemoMuxModule::getSupportedSdbIds()
{
    return { AFI_SDB_ADC_LEMO_MUX_CTRL };
}

void AdcLemoMuxModule::post_init()
{
    buildParams.reset();
    AdcLemoMuxBuildParams bp;
    RegOpVector r1;
    r1.RegRead32(REG32_ID, &bp.id);
    r1.RegRead16(REG_OUT_NUM, &bp.size);
    if (regOpExecRebased(r1)) {
        if(!getSupportedSdbIds().contains(static_cast<AFI_SDB_ID>(bp.id)))
            return;
        RegOpVector r2;
        QVector<quint64> arr(bp.size, 0);
        for(auto i=0; i<bp.size; ++i)
            r2.RegRead64(REG64_NAME_BASE+i*4, &arr[i]);
        if (regOpExecRebased(r2)) {
            for(auto i=0; i<bp.size; ++i) {
                auto n = QString::fromLatin1(
                                reinterpret_cast<const char*>(&arr[i]), 8).trimmed();
                n = n.mid(0, n.indexOf(char(0)));
                bp.names << n;
            }
            buildParams = bp;
        }
    }
}

bool AdcLemoMuxModule::writeConfig(const AdcLemoMuxCtrlConfig &cfg)
{
    if (!buildParams || !buildParams->isValid())
        return {};

    RegOpVector r;
    quint16 v = ((cfg.in & 0x1) << 8) | (cfg.out & 0x7);
    r.RegWrite16(REG_CONTROL, v);
    return regOpExecRebased(r);
}
