#include "CrosspointSwitchModule.h"

#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

#include "CrosspointSwitchConfig.h"

enum {
    REG32_ID        = 0x0,
    REG_IN_SIZE     = 0x4,
    REG_OUT_SIZE    = 0x5,
    REg_OUT_ARRAY   = 0x6,
};

QList<AFI_SDB_ID> CrosspointSwitchModule::getSupportedSdbIds()
{
    return { AFI_SDB_CROSSPOINT_SWITCH };
}

void CrosspointSwitchModule::post_init()
{
    buildParams.reset();
    CrosspointSwitchBuildParams bp;
    RegOpVector r;
    r.RegRead32(REG32_ID, &bp.id);
    r.RegRead16(REG_IN_SIZE, &bp.inSize);
    r.RegRead16(REG_OUT_SIZE, &bp.outSize);
    r.RegRead16(REg_OUT_ARRAY, &bp.outArrayAddr);
    if (regOpExecRebased(r)) {
        if(bp.isValid())
            buildParams = bp;
    }
}

bool CrosspointSwitchModule::writeConfig(const CrosspointSwitchConfig &cfg)
{
    if (!buildParams || !buildParams->isValid())
        return {};

    RegOpVector r;
    for(quint16 i=0; i<buildParams->outSize; ++i)
        r.RegWrite16(buildParams->outArrayAddr+i, cfg.get(i, buildParams->outSize));
    return regOpExecRebased(r);
}
