/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DEVICEMODULEMAPPER_H
#define DEVICEMODULEMAPPER_H

#include <QObject>
#include <QMap>
#include <QSharedPointer>

#include "AbstractDeviceModule.h"
#include "MregBusInterface.h"
#include "base/DeviceIndex.h"
#include "base/SdbModuleIndex.h"

enum AFI_SDB_ID : uint32_t;

class QtMregDevice;

class DeviceModuleMapper : public QObject
{
    Q_OBJECT
public:
    DeviceModuleMapper(const DeviceIndex &deviceIndex, QtMregDevice *qtMregDevice, QObject *parent)
        : QObject{parent}, deviceIndex(deviceIndex), qtMregDevice(qtMregDevice) {}

    QSharedPointer<AbstractDeviceModule> insert(QSharedPointer<AbstractDeviceModule> p) {
        modules.push_back(p);
        return p;
    }
    QList<SdbDevice> getModuleList(const QList<AFI_SDB_ID> &match) const;
    QList<SdbModuleIndex> getModuleIndexes() const;
    QSharedPointer<AbstractDeviceModule> findModule(const SdbModuleIndex index);
    QSharedPointer<const AbstractDeviceModule> findModule(const SdbModuleIndex index) const;
    QList<QSharedPointer<AbstractDeviceModule>> findModules(AFI_SDB_ID id);
    QList<QSharedPointer<const AbstractDeviceModule>> findModules(AFI_SDB_ID id) const;

    template<class T, typename... Arguments>
    QMap<int, QSharedPointer<T>> createModules(Arguments... args) {
        QMap<int, QSharedPointer<T>> result;
        const QList<AFI_SDB_ID> &ids = T::getSupportedSdbIds();
        const QList<SdbDevice> &records = getModuleList(ids);
        int portIndex = 0;
        for (auto it = records.begin(); it != records.end(); ++it) {
            const auto &sdbDevice = *it;
            MregBusInterface bus(qtMregDevice, sdbDevice);
            auto module = QSharedPointer<T>(new T(bus, args...));
            insert(module);
            result.insert(portIndex++, module);
        }
        return result;
    }

    void initModules();
    void deinitModules();

private:

    DeviceIndex deviceIndex;
    QtMregDevice * const qtMregDevice;
    QList<QSharedPointer<AbstractDeviceModule>> modules;
};

#endif // DEVICEMODULEMAPPER_H
