/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "FlashDevImpl.h"
#include "base/DeviceIndex.h"
#include "device-modules/AbstractEasyDeviceModule.h"
#include "regio/regop.h"

static const uint MAX_PART_SIZE = 256;

DeviceIndex FlashDevImpl::getDeviceIndex() const
{
    return devModule->getDeviceIndex();
}

bool FlashDevImpl::devReadReg(int regAddr, uint16_t *result)
{
    RegOpVector r;
    r.RegRead16(regAddr, result);
    return devModule->regOpExecRebased(r);
}

bool FlashDevImpl::devWriteReg(int regAddr, quint16 data)
{
    RegOpVector r;
    r.RegWrite16(regAddr, data);
    return devModule->regOpExecRebased(r);
}

bool FlashDevImpl::devWriteReg32(int regAddr, quint32 data)
{
    RegOpVector r;
    r.RegWrite32(regAddr, data);
    return devModule->regOpExecRebased(r);
}

bool FlashDevImpl::devBlkWriteReg(uint count, const uint *addr, uint16_t *data)
{
    RegOpVector opVec;
    uint written = 0;
    while(written != count) {
        uint sizeToWrite = qMin(count-written, MAX_PART_SIZE);
        for(uint w=0; w<sizeToWrite; ++w)
            opVec.RegWrite16(addr[written+w], data[written+w]);

        if (!devModule->regOpExecRebased(opVec))
            return false;

        written += sizeToWrite;
    }
    return true;
}

bool FlashDevImpl::devBlkReadReg(uint count, const uint *addr, uint16_t *data)
{
    RegOpVector opVec;
    uint read = 0;
    while(read != count) {
        uint sizeToRead = qMin(count-read, MAX_PART_SIZE);
        for(uint w=0; w<sizeToRead; ++w)
            opVec.RegRead16(addr[read+w], &data[read+w]);

        if (!devModule->regOpExecRebased(opVec))
            return false;

        read += sizeToRead;
    }
    return true;
}

QString FlashDevImpl::getPrefix() const
{
    return getDeviceIndex().getIdent();
}
