#ifndef FLASHPROGMODULE_H
#define FLASHPROGMODULE_H

#include <optional>

#include "device-modules/BaseDeviceModule.h"

class FlashDevTask;
class FlashDevImpl;
struct FlashProgStatus;
struct FlashIdData;
struct FlashImage;
enum class FlashOp;

namespace mlink { class RegOpVector; }

class FlashProgModule : public QObject, public BaseDeviceModule<FlashProgModule>
{
    Q_OBJECT
public:
    explicit FlashProgModule(const MregBusInterface &bus, QObject *parent);
    ~FlashProgModule() override;

    static QList<AFI_SDB_ID> getSupportedSdbIds();

    virtual void post_init() override;
    DeviceModuleCommandResult runCommand(const DeviceModuleCommand &command) override;

    std::optional<FlashProgStatus> readStatus();
    bool readStatus(FlashProgStatus *result);

    bool abortTask();
    void configureFlash(bool supportAddr4bMode);
    bool isConfigured() const;

    // async interface
    std::optional<FlashDevTask> getTasks() const;
    std::optional<FlashDevTask> getFinishedTasks() const;
    bool runTasks();
    bool isReady() const;
    bool submitTask(const FlashImage &image, FlashOp op);
    void setSingleCheckMode(bool v) const;

    bool softwareReboot();

signals:
    void taskFinished(const FlashDevTask &task);

private:
    static constexpr const bool readoutSavedTime = false;
    FlashDevImpl * const flashDev;
    FlashIdData * const flashIdData;
};

#endif // FLASHPROGMODULE_H
