/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "HrbModuleConfigConverter.h"
#include "base/AbstractConfigConverter.h"
#include "device-modules/Hrb/HrbModuleConfig.h"

namespace {
constexpr const char *k_latency = "latency";
constexpr const char *k_window = "window";
constexpr const char *k_minHitDuration = "minHitDuration";
constexpr const char *k_thr = "thr";
} // namespace

template<>
bool AbstractConfigConverter<HrbModuleConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<HrbModuleConfig>::toJsonObject(const HrbModuleConfig &in)
{
    QVariantMap m;
    m[k_latency] = QVariant::fromValue(in.latency);
    m[k_window] = QVariant::fromValue(in.window);
    m[k_minHitDuration] = QVariant::fromValue(in.minHitDuration);
    m[k_thr] = ConfigConverterUtil::toJsonValue(in.thr);
    return fromVariantMap(m);
}

template<>
HrbModuleConfig AbstractConfigConverter<HrbModuleConfig>::fromJsonObject(const QJsonObject &in)
{
    HrbModuleConfig out;
    update_value(in, k_latency, &out.latency);
    update_value(in, k_window, &out.window);
    update_value(in, k_minHitDuration, &out.minHitDuration);
    out.thr = ConfigConverterUtil::toMap<int, double>(in[k_thr]);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<HrbModuleConfig>::toJsonValue(const HrbModuleConfig &in)
{
    return toJsonObject(in);
}

template<>
HrbModuleConfig AbstractConfigConverter<HrbModuleConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<HrbModuleConfig>::registerSubTypes()
{}
