
#include "RunLogicStatus.h"
#include "base/TaiTime.h"
#include "device-modules/RegWrTimeTai.h"
#include "sdb/AfiSdbId.h"

#include <QStringList>

RunLogicStatus& RunLogicStatus::operator-=(const RunLogicStatus& r)
{
    currentRunTime -= r.currentRunTime;
    currentEventCount -= r.currentEventCount;
    currentTimeRaw = RegWrTaiTime::toRaw(currentTime() - r.currentTime());
    return *this;
}

bool RunLogicStatus::isValid() const
{
    return id == AFI_SDB_RUN_LOGIC;
}

TaiTime RunLogicStatus::startTime() const
{
    return RegWrTaiTime::fromRaw(startTimeRaw);
}

TaiTime RunLogicStatus::finishTime() const
{
    return RegWrTaiTime::fromRaw(finishTimeRaw);
}

TaiTime RunLogicStatus::currentTime() const
{
    return RegWrTaiTime::fromRaw(currentTimeRaw);
}

RunLogicStatus::State RunLogicStatus::getState() const
{
    switch (state & 0x3) {
    case 0: return State::Idle;
    case 1: return State::Running;
    case 2: return State::Finished;
    case 3: return State::Running;
    }
    return State::Idle;
}

QString RunLogicStatus::getStateStr() const
{
    switch (getState()) {
    case State::Idle: return "Idle";
    case State::Running: return "Run";
    case State::Finished: return "Finished";
    }
    return {};
}

QString RunLogicStatus::getFullText() const
{
    QStringList sl;
    sl << "state: " + QString("%1").arg(state, 4, 16, QChar('0')) + " " + getStateStr();
    //if (getState() != State::Idle)
        sl << "  started: " + startTime().toString();
    //if (getState() == State::Finished)
    sl << "  finished: " + finishTime().toString();
    sl << "events: " + QString("%L1").arg(currentEventCount);
    sl << "runtime: " + QString("%L1 s").arg(currentRunTime / 1e3);
    sl << "now: " + currentTime().toString();
    return sl.join("\n");
}
