/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TRIGCTRLSTATCONFIG_H
#define TRIGCTRLSTATCONFIG_H

#include <QMetaType>

class TrigCtrlStatConfig
{
public:
    enum TrigSource : quint32 {
        TrigSourceDigThr = 0,
        TrigSourceAnalogThr = 1,
        TrigSourceVxsPulse = 2,
        TrigSourceFelinkVxs = 3,
        TrigSourceFelinkFp = 4,
        TrigSourceExtPulse = 5,
        TrigSourceExtPulse2 = 6,
        TrigSourceExtPulse3 = 7,
        TrigSourceExtPulse4 = 8,
        TrigSourcePeriodic = 14,
        TrigSourceRandom = 15
    };
    enum TrigPolarity { Positive = 0, Negative = 1};

    friend bool operator==(const TrigCtrlStatConfig &, const TrigCtrlStatConfig &) = default;
    bool isEnabled(TrigSource index) const {
        return source_mask & (1u << index);
    }
    void setEnabled(TrigSource index, bool state = true) {
        quint32 mask = 1u << index;
        if (state) source_mask |= mask;
        else source_mask &= ~mask;
    }

    bool isNegative(TrigSource index) const {
        return polarity_mask & (1u << index);
    }
    void setPolarity(TrigSource index, TrigPolarity state = Positive) {
        quint32 mask = 1u << index;
        if (state) polarity_mask |= mask;
        else polarity_mask &= ~mask;
    }
    void mergeLocalData(const TrigCtrlStatConfig &r)
    {
        dig_thr_channel_en = r.dig_thr_channel_en;
        analog_thr_channel_en = r.analog_thr_channel_en;
    }

    double trigTimerFreq = 1000;  // Hz
    double randomPulserFreq = 1000; // Hz
    double randomPulserMinPeriod = 10e-6; // seconds
    double programmedDeadTime = 1e-6; // seconds
    quint16 analogThrDelay = 1; // in 8ns step
    quint16 digThrDelay = 1; // in 8ns step

    quint32 source_mask = 0;
    quint32 polarity_mask = 0;

    quint64 dig_thr_channel_en = ~0ull;
    quint64 analog_thr_channel_en = ~0ull;
};
Q_DECLARE_METATYPE(TrigCtrlStatConfig)

#endif // TRIGCTRLSTATCONFIG_H
