#ifndef TTLIOCONFIG_H
#define TTLIOCONFIG_H

#include <QVector>
#include <QMetaType>

enum class TtlIoInput
{
    Trigger          = 0x00,
    Busy             = 0x01,
    ActiveLowBusy    = 0x02,
    EventNumberReset = 0x03,
};

enum class TtlIoOutput
{
    AcceptedTrigger = 0x00,
    BlockedTrigger  = 0x01,
    Busy            = 0x02,
    Pps             = 0x03,
    Pulse24ns       = 0x04,
    Pulse48ns       = 0x05,
    Logic0          = 0x06,
    Logic1          = 0x07,
};

typedef union reg_ttl_io_t {
    void setInput(TtlIoInput i);
    void setOutput(TtlIoOutput o);
    TtlIoInput getInput() const;
    TtlIoOutput getOutput() const;
    QString toString() const;

    uint16_t all;
    struct {
        uint16_t cfg: 15;
        uint16_t isOutput: 1;
    } bit;
} reg_ttl_io_t;
Q_DECLARE_METATYPE(reg_ttl_io_t)

inline bool operator==(const reg_ttl_io_t &l, const reg_ttl_io_t &r)
{
    return l.all == r.all;
}

struct TtlIoConfig
{
    static QList<TtlIoInput> getInputs();
    static QList<TtlIoOutput> getOutputs();
    static QString getName(const TtlIoInput &type);
    static QString getName(const TtlIoOutput &type);
    static QString getPrettyName(const TtlIoInput &type);
    static QString getPrettyName(const TtlIoOutput &type);

    friend bool operator==(const TtlIoConfig &l, const TtlIoConfig &r) = default;
    friend bool operator!=(const TtlIoConfig &l, const TtlIoConfig &r) = default;

    QVector<reg_ttl_io_t> ttl_regs;
};
Q_DECLARE_METATYPE(TtlIoConfig)


QDebug operator<<(QDebug dbg, const TtlIoInput &r);
QDebug operator<<(QDebug dbg, const TtlIoOutput &r);
QDebug operator<<(QDebug dbg, const reg_ttl_io_t &r);
QDebug operator<<(QDebug dbg, const TtlIoConfig &r);
QDataStream &operator<<(QDataStream &out, const reg_ttl_io_t &r);
QDataStream &operator>>(QDataStream &in, reg_ttl_io_t &r);
QDataStream &operator<<(QDataStream &out, const TtlIoConfig &r);
QDataStream &operator>>(QDataStream &in, TtlIoConfig &r);

#endif // TTLIOCONFIG_H
