#include "TtlIoModule.h"

#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

#include "TtlIoConfig.h"

// Deserializers control
enum {
    REG_STATUS = 0x00,
    REG_CFG_BASE = 0x01,
};

QList<AFI_SDB_ID> TtlIoModule::getSupportedSdbIds()
{
    return {AFI_SDB_TTL_IO_CTRL};
}

void TtlIoModule::post_init()
{
    regRead16(REG_STATUS, &ttlCount);
}

TtlIoConfig TtlIoModule::readConfig()
{
    QVector<quint16> val(ttlCount);
    RegOpVector r;
    const quint16 cfgBase = REG_CFG_BASE;
    for (int i = 0; i < ttlCount; ++i)
        r.RegRead16(cfgBase + i, &val[i]);

    if (!regOpExecRebased(r))
        return {};

    TtlIoConfig cfg;
    cfg.ttl_regs.resize(ttlCount);
    for (int i = 0; i < ttlCount; ++i)
        cfg.ttl_regs[i].all = val[i];
    return cfg;
}

bool TtlIoModule::writeConfig(const TtlIoConfig &cfg)
{
    RegOpVector r = setConfig(cfg);
    return regOpExecRebased(r);
}

mlink::RegOpVector TtlIoModule::setConfig(const TtlIoConfig &cfg) const
{
    RegOpVector r;
    const quint16 cfgBase = REG_CFG_BASE;
    for (int i = 0; i < cfg.ttl_regs.size(); ++i) {
        if (i > ttlCount)
            break;
        r.RegWrite16(cfgBase + i, cfg.ttl_regs[i].all);
    }
    return r;
}
