/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef WAVERECCONFIG_H
#define WAVERECCONFIG_H

#include <QMap>
#include <QMetaType>

#include "AdcErc/AdcErcConfig.h"

class WaveRecConfig
{
public:
    enum ZsMode {ZsDisabled = 0, ZsAnlg = 1, ZsDig = 2};
    friend bool operator==(const WaveRecConfig &, const WaveRecConfig &) = default;
    void mergeLocalData(const WaveRecConfig &r)
    {
        chAdcEn = r.chAdcEn;
        chSignalInvert = r.chSignalInvert;
        chZsEn = r.chZsEn;
        chZsRising = r.chZsRising;
        chZsThr = r.chZsThr;
    }

    ZsMode zsMode = ZsDisabled;
    int adcPreWin = 2;
    int readCellNumber = 8;
    int latency = 24;
    int matchWin = 24;

    QMap<int, bool> chAdcEn;
    QMap<int, bool> chSignalInvert;
    QMap<int, bool> chZsEn;
    QMap<int, bool> chZsRising;
    QMap<int, int> chZsThr;

    SparseParams sparse;
};
Q_DECLARE_METATYPE(WaveRecConfig)

#endif // WAVERECCONFIG_H
