#ifndef TQDCANALYZER_H
#define TQDCANALYZER_H

#include <QObject>
#include <QByteArray>
#include <QElapsedTimer>
#include <QMap>

#include "base/DeviceIndex.h"

class QSemaphore;
class QTimer;

struct AnalyzerStat;
struct DecodedData;
class MpdDataDecoder;
struct RcRunId;

class TQDCAnalyzer : public QObject
{
    Q_OBJECT
public:
    explicit TQDCAnalyzer(QSemaphore *_analyzerSem,
                          QSemaphore *_histSem,
                          QSemaphore *_writerSem,
                          QSemaphore *_zmqSem,
                          QObject *parent = nullptr);

signals:
    void addLog(QString) const;
    void deviceDataUpdated(const DecodedData &data,
                           bool writeDecodedEnabled) const;
    void deviceStatsUpdated(const AnalyzerStat &stats) const;
    void rawDataUpdated(const QStringList &data);
    void gotStopEvent(quint32 stopEvNum);
    void runStarted(const RcRunId &) const;
    void runStoped(const RcRunId &) const;

public slots:
    void dataAccuired(const QByteArray &newData);
    void reset();
    void requestRawData(int events);
    void setRawDataEveryEv(bool everyEv);
    void invertUpdated(QMap<DeviceIndex, QVector<bool> > _invertMap);
    void set_can_skip(bool skip);
    void prepareForStop(quint32 evNum);
    void startWrite(QString, bool rawData);
    void stopWrite();

private slots:
    void sendStat();

private:
    void flushData();

    QSemaphore * const analyzerSem;// free space in analyzer queue
    QSemaphore * const histSem;// free space in histogram queue
    QSemaphore * const writerSem;// free space in writer queue (raw or decoded)
    QSemaphore * const zmqSem;// free space in 0MQ queue

    QTimer * const statTimer;

    bool writeDecodedEnabled = false;
    bool printBadEvent = true;
    QStringList lastErrStr;
    int clear_err_stack = 0;
    bool can_skip = true;
    QElapsedTimer noHistSpace;
    QElapsedTimer noWriterSpace;
    QElapsedTimer noZmqSpace;

    MpdDataDecoder * const decoder;
};

#endif // TQDCANALYZER_H
