/*
**    Copyright 2011-2022 Ivan Filippov
**    Copyright 2011-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef MLINKSTREAMRECEIVER_H
#define MLINKSTREAMRECEIVER_H

#include <QObject>
#include <QElapsedTimer>
#include <QVector>
#include <QMetaType>
#include <QSemaphore>
#include <QString>

#include "base/DeviceIndex.h"
#include "MStreamV2Client.h"
#include "PacketBuffer.h"
#include "ReceiverStat.h"

typedef QPair<int, int> LoopInterval;
class QTimer;
class QTime;
class RawCheck;
class QString;
class QHostAddress;
class PacketRingBuffer;

class MlinkStreamReceiver : public QObject
{
    Q_OBJECT
public:
    explicit MlinkStreamReceiver(DeviceIndex index, QSharedPointer<PacketRingBuffer> ringBuf, bool _bigFragId=false,
                                 quint16 hwBufSize=8, bool _doPrintStat = true);
    virtual ~MlinkStreamReceiver();
//    HwipSocketAddress getLocalAddress() const { return mstream.getLocalAddress(); }
//    HwipSocketAddress getPeerAddress() const { return mstream.getPeerAddress(); }

    void setDeleteFlag();
    bool hasDeleteFlag() const;

    QSharedPointer<PacketRingBuffer> sharedBuffer;

signals:
    void gotData(QVector<PacketBuffer> tasks);
    void stat_from_receiver_updated(const ReceiverStat &stat);
    void rxPortUpdated(quint16 p) const;
    void peerConnected();
    void peerDisconnected();

private slots:
    void deferredInit();
    void runLoop();
#if (! SEM_INTERACT)
    void sendWork();
#endif
public slots:
#if (! SEM_INTERACT)
    void releaseData(int shift);
#endif
    void deleteReceiver();
    void connectToHardware(bool force = true);
    void disconnectFromHardware();
    void hostDiscovered(const QHostAddress &address);

protected:
    ssize_t recvfrom(); // move it
        void printLoopStat(int loopCount, int loopTime);

    std::atomic<bool> deleteFlag{false};
    DeviceIndex deviceIndex;
    MStreamV2Client mstream;

    const bool bigFragId; // fragmentId is 16-bit length
    QElapsedTimer * const runLoopTime;
    QTimer * const runLoopTimer;
    RawCheck *const rc;
    QTimer * const sendWorkTimer;

    QVector<PacketBuffer> sendtasks;

    //    int dataSeq;
    const int RECV_LIMIT;
    std::vector<mmsghdr> msgsTt;
    std::vector<iovec> iovecsTt;
    AckList ackStack;

    ReceiverStat stat;

    int timeoutCnt;
    uint16_t myPort;
    int runTimeCheckPeriodCnt;
    bool doPrintStat;
};

#endif // MLINKSTREAMRECEIVER_H
